/*
This file is part of the IODEV (Input Output DEVices) component of BOCHS.

The IODEV component is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

The IODEV component is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the IODEV component; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/



#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <sys/time.h>

#include "iodev.h"

#define BX_PIT_XFER_MODE_LSB   10
#define BX_PIT_XFER_MODE_MSB   11
#define BX_PIT_XFER_MODE_16BIT 12

#define BX_PIT_STATE_ACTIVE        10
#define BX_PIT_STATE_PENDING_READ  11
#define BX_PIT_STATE_PENDING_WRITE 12


typedef struct {
  Bit8u      mode;
  Bit8u      xfer_mode;
  Bit8u      state;
  Bit16u     latch_value;
  Bit8u      xfer_bytes_remaining;
  Bit16u     counter_max;
  Boolean    bcd_mode;
  } bx_pit_t;

static bx_pit_t timer0;


  Bit8u
bx_pit_handler(Bit8u call_reason,
               Bit8u address_space, Bit32u address, Bit8u value)
{
  switch (call_reason) {
    case BX_CALL_REASON_TIMER:
      bx_trigger_irq(0);
      break;
    }
}

  void
bx_pit_int_handler(int vector)
{
  fprintf(stderr, "^");
  bx_outp(0x20, 0x20);
}

  void
bx_init_pit(void)
{
  sigset_t sig_mask_set;
  int ret;
  struct itimerval new_timer_value;
  struct sigaction action;

  timer0.mode        = 3;  /* square wave */
  timer0.xfer_mode   = BX_PIT_XFER_MODE_16BIT;
  timer0.state       = BX_PIT_STATE_ACTIVE;
  timer0.latch_value = 0;  /* don't care */
  timer0.xfer_bytes_remaining = 0; /* don't care */
  timer0.counter_max = 0;  /* 0xFFFF + 1 : (1193200 / 65535 = 18.2Hz) */
  timer0.bcd_mode    = 0;  /* binary counting mode */


#ifndef HAVE_SIGACTION
  signal(SIGVTALRM, bx_master_event_handler);
#else /* use POSIX sigaction() instead */

  /* set signal action:  auto restart system calls,  handler remains
     installed after invokation of handler */

  /* get current signal handling features */
  ret = sigaction(SIGVTALRM, NULL, &action);
  if (ret != 0) {
    perror("bx_init_pic()");
    exit(0);
    }

  /* set up signal handler function */
  action.sa_handler = bx_master_event_handler;

  /* automatic system call restart when interrupted by signal */
#if defined SA_RESTART
  action.sa_flags |= SA_RESTART;
#else
  siginterrupt(SIGVTALRM, 0);
#endif

  /* reinstate handler function upon exit from handler */
#if defined SA_ONESHOT
  action.sa_flags &=  ~SA_ONESHOT;   /* linux */
#elif defined SA_RESETHAND
  action.sa_flags &= ~SA_RESETHAND; /* Solaris2.? */
#else
  /* ---                                    --- */
  /* --- insert code for your platform here --- */
  /* ---                                    --- */
#endif
/* ifdef SA_ONESHOT */

  ret = sigaction(SIGVTALRM, &action, NULL);
  if (ret != 0) {
    perror("bx_init_pit()");
    exit(0);
    }

#endif /* #ifndef HAVE_SIGACTION */



  /* fire off a SIGVTALRM every interval value seconds */
  new_timer_value.it_interval.tv_sec  = 0;
  new_timer_value.it_interval.tv_usec = 54925; /* 54.925mS = 18.2Hz */
  /* starting value = interval value */
  new_timer_value.it_value = new_timer_value.it_interval;

  setitimer(ITIMER_VIRTUAL, &new_timer_value, NULL);
}

